import React, { useMemo, useState } from 'react';
import { cloneDeep } from 'lodash';
import useForceUpdate from 'use-force-update';
import { ValidationError } from 'json-server-orm';
import { log } from './common/log';
import { env } from './common/env';
// FIXME Warning: A component is changing an uncontrolled input of type checkbox to be controlled. Input elements should not switch from uncontrolled to controlled (or vice versa). Decide between using a controlled or uncontrolled input element for the lifetime of the component. More info: https://fb.me/react-controlled-components
export function EditableRow({ table, item: editItem, onSubmit, onCancel }) {
    const db = env.db;
    const validation = env.validation;
    const dbTable = useMemo(() => db[table], [table]);
    const fields = useMemo(() => validation.getOwnFields(table), [table]);
    const [item] = useState(editItem ? cloneDeep(editItem) : validation.getDefaultItem(table));
    const [error, setError] = useState();
    const handleClick = async () => {
        try {
            log('[EditableRow.handleClick]', item);
            const newItem = editItem ? await dbTable.update(item) : await dbTable.add(item);
            log('[EditableRow.handleClick] newItem', newItem);
            onSubmit && onSubmit(newItem);
        }
        catch (err) {
            if (err.response)
                err = new ValidationError(err.response.data.message, err.response.data.field);
            if (err instanceof ValidationError && err.field)
                setError(err);
            else
                alert(err.message);
        }
    };
    return React.createElement("tr", { className: 'row-edit' },
        React.createElement("td", { className: 'pad' },
            React.createElement("i", { className: 'icon submit', onClick: handleClick }),
            React.createElement("i", { className: 'icon cancel', onClick: () => onCancel && onCancel() })),
        fields.map(f => (React.createElement("td", { key: f },
            React.createElement(EditableField, { table: table, item: item, f: f, error: error })))));
}
function EditableField({ table, item, f, error }) {
    if (f === 'id')
        return React.createElement("span", { className: 'pad' }, item[f] || '');
    const validation = env.validation;
    const ps = useMemo(() => validation.getOwnProperties(table), [table]);
    const forceUpdate = useForceUpdate();
    const handleChange = (ev) => {
        // console.log('handleChange', ev.target.value);
        const type = ps[f].type;
        switch (type) {
            case 'number':
            case 'integer':
            case 'string':
            case 'null':
            case 'object':
            case 'array':
                item[f] = ev.target.value;
                break;
            case 'boolean':
                item[f] = ev.target.checked;
                break;
        }
        forceUpdate();
    };
    const handleBlur = (ev) => {
        // console.log('handleBlur', ev.target.value);
        const f = ev.target.name;
        const type = ps[f].type;
        const rawvalue = ev.target.value;
        let newvalue;
        switch (type) {
            case 'number':
                newvalue = parseFloat(rawvalue);
                break;
            case 'integer':
                newvalue = parseInt(rawvalue);
                break;
            case 'string':
                newvalue = rawvalue;
                break;
            case 'boolean':
                newvalue = ev.target.checked;
                break;
            case 'object':
            case 'array':
                newvalue = JSON.parse(rawvalue);
                break;
            case 'null':
                newvalue = null;
                break;
        }
        item[f] = newvalue;
        forceUpdate();
    };
    const type = ps[f].type;
    const hasError = error?.field === f;
    const errUI = hasError ? React.createElement("span", { className: "error" }, error?.message) : React.createElement(React.Fragment, null);
    const className = 'field type-' + type + (hasError ? ' error' : '');
    switch (type) {
        case 'number':
        case 'integer':
        case 'string':
            return React.createElement(React.Fragment, null,
                React.createElement("input", { name: f, type: 'text', className: className, maxLength: ps[f].maxLength, minLength: ps[f].minLength, value: item[f], onChange: handleChange, onBlur: handleBlur }),
                errUI);
        case 'boolean':
            return React.createElement("input", { name: f, type: 'checkbox', className: className, checked: item[f], onChange: handleChange, onBlur: handleBlur });
        case 'object':
        case 'array':
            return React.createElement(React.Fragment, null,
                React.createElement("textarea", { name: f, className: className, maxLength: ps[f].maxLength, minLength: ps[f].minLength, value: typeof item[f] === 'string' ? item[f] : JSON.stringify(item[f], null, 2), onChange: handleChange, onBlur: handleBlur }),
                errUI);
        case 'null':
            return React.createElement("span", { className: 'readonly' }, "null");
            break;
    }
}
